package game;

import haxe.ds.StringMap;
import sk.thenet.plustd.FM;
import sk.thenet.gamestd.input.*;
import sk.thenet.gamestd.bmp.*;
import sk.thenet.gamestd.*;

class GSGame extends StaterState {
	public static inline var GC_NORMAL:Int = 0;
	public static inline var GC_MAN_CHOICE:Int = 1;
	public static inline var GC_MAN_CHOSEN:Int = 2;
	public static inline var GC_COLLECT:Int = 3;
	public static inline var GC_VICTORY:Int = 4;
	public static inline var GC_DEATH:Int = 5;
	public static inline var GC_DEATH_COLLECT:Int = 6;
	
	public static var BAT_MAX:Int = 7200;
	public static var HP_MAX:Int = 72;
	
	public var tree:Tree;
	public var lvl:Level = null;
	public var phsx:Physics;
	public var battery:Array<Int>;
	public var damage:Array<Int>;
	public var pres:Array<Bool>;
	public var control:Int = 1;
	public var levelstates:Array<Array<Bool>>;
	public var col_red:Int = 0;
	public var col_green:Int = 0;
	public var col_blue:Int = 0;
	
	public var cmc_sel:Int = 0;
	public var cmc_phase:Int = 0;
	public var cmc_level:Int = 0;
	
	public var vict_r:Array<Bool>;
	public var vict_bd:BitmapData;
	
	public function new(){
		super("game");
	}
	
	override public function fTo(s:Stater, id:StringMap<Dynamic> = null):Void {
		tree = new Tree();
		battery = [
			BAT_MAX - 3600,
			BAT_MAX - 3250,
			BAT_MAX - 2300
		];
		damage = [
			HP_MAX - 40,
			HP_MAX - 22,
			HP_MAX - 36
		];
		pres = [
			true,
			true,
			true
		];
		Level.resetLevels();
		var to:String = "hub";
		if (id != null && id.exists("lvl")) to = id.get("lvl");
		//to = "shrooms";
		//control = GC_NORMAL;
		switchLevel(to);
		lvl.man.pe.xp = 16 * 5;
		lvl.man.pe.yp = 16 * 18 + 8;
		lvl.man.invisible = true;
	}
	
	override public function fFrame(s:Stater):Void {
		if (control == GC_MAN_CHOSEN){
			cmc_phase++;
			if (cmc_phase >= 60){
				pres[cmc_sel] = false;
				control = GC_NORMAL;
				cmc_phase = 0;
				lvl.man.invisible = false;
				lvl.man.pe.xp = 16 * (5 + cmc_sel) + 8;
			}
		}
		if ((control == GC_NORMAL || control == GC_MAN_CHOICE) && lvl.id == "hub" && tree.prog >= 30){
			control = GC_VICTORY;
			cmc_phase = 0;
		}
		if (control == GC_NORMAL && (battery[cmc_sel] <= 0 || damage[cmc_sel] <= 0)){
			Assets.sDeathPlayer.play();
			control = GC_DEATH;
			cmc_phase = 0;
		}
		if (control == GC_VICTORY){
			if (cmc_phase < 200 && cmc_phase % 3 == 0)
				tree.collect(Math.random() > .5 ? 1 : 0);
			var v = s.bd.getFullVector();
			var i:Int = 0;
			if (cmc_phase == 0){
				vict_r = [
					for (i in 0...600){
						Math.random() > .5;
					}
				];
				vict_bd = new BitmapData(320, 240, 0);
				Assets.fGlow.render(vict_bd, "congratulations\nyou have saved the Tree and thus restored\nbalance in the Universe\n\n" + 
					"thank you for playing\nhARBOuR, a game by Aurel B$l% (thenet.sk)\nspecial thanks to Lily", 16, 140, 42);
			} else {
				vict_r.unshift(Math.random() > .5);
				vict_r.pop();
			}
			for (y in 0...240){
				for (x in 0...320){
					var xd:Int = x - 160;
					var yd:Int = y - 120;
					if (xd < 0) xd = -xd;
					if (yd < 0) yd = -yd;
					var treecol:UInt = 0xFFFFFFFF ^ (2 * (xd + yd) < cmc_phase + 100 ? 0xFFFFFF & vict_bd.get(x, y) : 0);
					if (x >= 128 && x < 192 && y >= 56 && y < 120){
						treecol = 0xF0F0F0 ^ tree.cbd.get(x - 128, y - 56);
						if (treecol & 0xFF000000 == 0){
							treecol = 0xFFFFFFFF;
						}
					}
					v[i] = (xd + yd < cmc_phase ? Assets.palette[vict_r[xd + yd] ? 10 : 5]
						& treecol : v[i]);
					i++;
				}
			}
			cmc_phase++;
			s.bd.setFullVector(v);
			return;
		}
		if (control == GC_DEATH){
			var xs:Int = FM.floor(Math.random() * 320);
			var ys:Int = FM.floor(Math.random() * 240);
			var xe:Int = FM.floor(Math.random() * 320);
			var ye:Int = FM.floor(Math.random() * 240);
			if (xs > xe){
				var t = xs;
				xs = xe;
				xe = xs;
			}
			if (ys > ye){
				var t = ys;
				ys = ye;
				ye = ys;
			}
			var v = s.bd.getFullVector();
			var cb:Int = FM.floor(Math.random() * 24);
			for (y in ys...ye){
				for (x in xs...xe){
					var i:Int = x + y * 320;
					v[i] = (y < cmc_phase - 100 ? Assets.palette[17] : v[i] ^ (1 << cb));
				}
			}
			s.bd.setFullVector(v);
			cmc_phase++;
			if (cmc_phase >= 350){
				switchLevel("hub");
				control = GC_DEATH_COLLECT;
				cmc_phase = 0;
				lvl.man.pe.xp = 16 * 5;
				lvl.man.pe.yp = 16 * 18 + 8;
				lvl.man.invisible = true;
			}
			return;
		}
		lvl.tick();
		phsx.tickAll();
		lvl.render(s.bd);
		s.bd.copyPixels(phsx.dbd, 0, 0, 0, 0, 320, 240);
		cmc_level = 0;
		if (control == GC_NORMAL || control == GC_MAN_CHOICE || control == GC_MAN_CHOSEN || control == GC_COLLECT || control == GC_DEATH_COLLECT){
			var gx:Int = 0;
			var gy:Int = 0;
			if (control == GC_MAN_CHOICE){
				if (!pres[0] && !pres[1] && !pres[2]){
					Assets.fWhite.render(s.bd, "game over ...", 200, 230 - lvl.camyi, 100);
					gx = 320;
					gy = 240;
				} else {
					Assets.fWhite.render(s.bd, "<> choose a huMAN\nA  start\n\nhuMAN #" + (cmc_sel + 1), 200, 230 - lvl.camyi, 100);
					gx = 195;
					gy = 195;
				}
			} else if (control == GC_MAN_CHOSEN){
				Assets.fWhite.render(s.bd, "<> movement\n^  jump\n\\^ climb, swim\nA  shoot", 200, 230 - lvl.camyi, 100);
				gx = FM.floor(200 - cmc_phase / 60 * 200);
				gy = FM.floor(200 - cmc_phase / 60 * 200);
			} else if (control == GC_COLLECT || control == GC_DEATH_COLLECT){
				if (control == GC_DEATH_COLLECT){
					gx = 320;
					gy = 240;
				}
				if (cmc_phase == 0){
					if (col_red > 0){
						col_red--;
						cmc_phase = 16;
						tree.collect(0);
					} else if (col_green > 0){
						col_green--;
						cmc_phase = 16;
						tree.collect(1);
					} else {
						control = (control == GC_COLLECT ? GC_NORMAL : GC_MAN_CHOICE);
					}
				} else {
					cmc_phase--;
				}
			} else if (lvl.id == "hub"){
				if (lvl.man.pe.xp >> 4 >= 17 && lvl.man.pe.yp >> 4 >= 11 && lvl.man.pe.yp >> 4 < 13){
					Assets.fWhite.render(s.bd, "crystal grotto\n\nA  enter", 200, 230 - lvl.camyi, 100);
					cmc_level = 1;
				} else if (lvl.man.pe.xp >> 4 >= 17 && lvl.man.pe.yp >> 4 >= 8 && lvl.man.pe.yp >> 4 < 10){
					Assets.fWhite.render(s.bd, "shroom waters\n\nA  enter", 200, 230 - lvl.camyi, 100);
					cmc_level = 2;
				} else {
					Assets.fWhite.render(s.bd, "<> movement\n^  jump\n\\^ climb, swim\nA  shoot", 200, 230 - lvl.camyi, 100);
				}
			} else if (lvl.id == "grotto"){
				if (lvl.man.pe.xp >> 4 >= 29 && lvl.man.pe.yp >> 4 >= 21 && lvl.man.pe.yp >> 4 < 23){
					Assets.fGlow.render(s.bd, "press A to go back to the pod", 16, 212, 100);
					cmc_level = 3;
				}
			}
			s.bd.copyPixels(Assets.aGUI, gx, gy, 0, 0, 82, 30);
			var bat:Int = FM.floor((battery[cmc_sel] / BAT_MAX) * 55);
			s.bd.copyPixels(Assets.aGUI, gx + 15 + bat, gy + 2, 82 + bat, 16, 55 - bat, 9);
			var hp:Int = FM.floor((damage[cmc_sel] / HP_MAX) * 55);
			s.bd.copyPixels(Assets.aGUI, gx + 25 + hp, gy + 14, 82 + hp, 25, 55 - hp, 9);
		}
		if (control == GC_NORMAL && lvl.id != "hub"){
			if (battery[cmc_sel] > 0) battery[cmc_sel]--;
			if (battery[cmc_sel] <= 0){
				// death
			}
		}
	}
	
	override public function fFrom(s:Stater):Void {
		tree = null;
		lvl = null;
		phsx = null;
	}
	
	override public function fKUp(s:Stater, k:Int):Void {
		switch (control){
			case GC_NORMAL:
			lvl.kUp(k);
			case GC_MAN_CHOICE:
			switch (k){
				case Keyboard.K_LEFT | Keyboard.K_RIGHT:
				Assets.sSelect.play();
				cmc_sel += (k == Keyboard.K_LEFT ? 2 : 1);
				cmc_sel %= 3;
				case Keyboard.K_A:
				if (pres[cmc_sel]){
					Assets.sStartup.play();
					control = GC_MAN_CHOSEN;
					cmc_phase = 0;
				}
			}
		}
	}
	
	override public function fKDown(s:Stater, k:Int):Void {
		if (control == GC_NORMAL){
			if (cmc_level != 0 && k == Keyboard.K_A){
				if (cmc_level == 1) switchLevel("grotto");
				else if (cmc_level == 2) switchLevel("shrooms");
				else {
					switchLevel("hub");
					if (col_red > 0 || col_green > 0){
						control = GC_COLLECT;
					}
				}
				cmc_level = 0;
				return;
			}
			lvl.kDown(k);
		}
	}
	
	public function switchLevel(to:String){
		if (lvl != null){
			lvl.ent = [];
		}
		lvl = Level.getLevel(to);
		phsx = new Physics();
		phsx.lvl_w = lvl.width;
		phsx.lvl_h = lvl.height;
		phsx.lvl = lvl.colmap;
		lvl.init(lvl, phsx);
		for (ie in lvl.init_ent){
			ie(lvl, phsx);
		}
		lvl.camx = lvl.man.pe.xp - 160;
		lvl.camy = lvl.man.pe.yp - 120;
	}
}