package game;

import sk.thenet.gamestd.model.*;
import sk.thenet.gamestd.bmp.*;

class Physics {
	public static inline var P_MASK_TILE:Int = 0xFF;
	public static inline var P_LVL_EMPTY:Int = 0;
	public static inline var P_LVL_FULL:Int = 1;
	public static inline var P_LVL_SLOPE_11_NW:Int = 2;
	public static inline var P_LVL_SLOPE_11_NE:Int = 3;
	public static inline var P_LVL_SLOPE_11_SW:Int = 4;
	public static inline var P_LVL_SLOPE_11_SE:Int = 5;
	
	public static inline var P_MASK_GRAVITY:Int = 0xFF00;
	public static inline var P_LVL_WATER:Int = 0x100;
	public static inline var P_LVL_LADDER:Int = 0x200;
	
	public static inline var P_MASK_ONEWAY:Int = 0xFF0000;
	public static inline var P_LVL_NO_LEFT:Int = 0x10000;
	public static inline var P_LVL_NO_RIGHT:Int = 0x20000;
	public static inline var P_LVL_NO_BOTTOM:Int = 0x40000;
	public static inline var P_LVL_NO_TOP:Int = 0x80000;
	
	public var ent:Array<PhysicsEntity>;
	public var lvl:Array<Int>;
	public var lvl_w:Int;
	public var lvl_h:Int;
	
	public var dbd:BitmapData;
	
	public function new(){
		ent = [];
		dbd = new BitmapData(320, 240, 0);
	}
	
	public function debugRender(bd:BitmapData):Void {
		var i:Int = 0;
		for (y in 0...lvl_h){
			for (x in 0...lvl_w){
				if (lvl[i] == 1){
					bd.fillRect((x << 4), (y << 4), 16, 16, 0xFF00AA00);
					bd.fillRect((x << 4) - 4, (y << 4) - 7, 25, 1, 0xFFAA0000);
					bd.fillRect((x << 4) - 4, (y << 4) - 7, 1, 28, 0xFFAA0000);
					bd.fillRect((x << 4) - 4, (y << 4) + 20, 25, 1, 0xFFAA0000);
					bd.fillRect((x << 4) + 20, (y << 4) - 7, 1, 28, 0xFFAA0000);
				}
				i++;
			}
		}
	}
	
	public function tickAll():Void {
		ent = ent.filter(function(a:PhysicsEntity){
			return !a.remove;
		});
		for (e in ent){
			tickEntity(e);
		}
	}
	
	public inline function tickEntity(e:PhysicsEntity):Void {
		if (e.type == PhysicsEntity.PET_NONE) return;
		e.colx1 = false;
		e.colx2 = false;
		e.coly1 = false;
		e.coly2 = false;
		var boundx1:Int = 0;
		var boundx2:Int = 0;
		var boundy1:Int = 0;
		var boundy2:Int = 0;
		switch (e.type){
			case PhysicsEntity.PET_BULLET:
			boundx1 = 2;
			boundx2 = 3;
			boundy1 = 2;
			boundy2 = 3;
			case _:
			boundx1 = 4;
			boundx2 = 5;
			boundy1 = 7;
			boundy2 = 5;
		}
		var prog = new Bresenham(e.xp, e.yp, e.xp + e.xv, e.yp + e.yv);
		var lb = new IntPoint2D(e.xp, e.yp);
		var lp = new IntPoint2D(e.xp, e.yp);
		var cp = new IntPoint2D(e.xp, e.yp);
		var colx = false;
		var coly = false;
		for (p in prog){
			var dx:Int = p.x - lb.x;
			var dy:Int = p.y - lb.y;
			lb = new IntPoint2D(p.x, p.y);
			cp = new IntPoint2D(lp.x + (!colx ? dx : 0), lp.y + (!coly ? dy : 0));
			var nxt:Int = cp.x >> 4;
			var nyt:Int = cp.y >> 4;
			for (ty in (nyt - 1)...(nyt + 2)){
				for (tx in (nxt - 1)...(nxt + 2)){
					if (tx < 0 || tx >= lvl_w || ty < 0 || ty >= lvl_h) continue;
					if (lvl[tx + ty * lvl_w] & P_MASK_TILE == 0) continue;
					
					var chk_x1 = lvl[tx + ty * lvl_w] & P_LVL_NO_LEFT == 0 && (tx <= 0
						|| (lvl[tx - 1 + ty * lvl_w] & P_MASK_TILE == 0
							|| lvl[tx - 1 + ty * lvl_w] & P_MASK_ONEWAY != 0));
					var chk_x2 = lvl[tx + ty * lvl_w] & P_LVL_NO_RIGHT == 0 && (tx >= lvl_w - 1
						|| (lvl[tx + 1 + ty * lvl_w] & P_MASK_TILE == 0
							|| lvl[tx + 1 + ty * lvl_w] & P_MASK_ONEWAY != 0));
//							&& lvl[tx - 1 + ty * lvl_w] & P_LVL_NO_LEFT == 0));
					var chk_y1 = lvl[tx + ty * lvl_w] & P_LVL_NO_TOP == 0 && (ty <= 0
						|| (lvl[tx + (ty - 1) * lvl_w] & P_MASK_TILE == 0
							|| lvl[tx + (ty - 1) * lvl_w] & P_MASK_ONEWAY != 0));
						// && lvl[tx - 1 + ty * lvl_w] & P_LVL_NO_BOTTOM == 0));
					var chk_y2 = lvl[tx + ty * lvl_w] & P_LVL_NO_BOTTOM == 0 && (ty >= lvl_h - 1
						|| (lvl[tx + (ty + 1) * lvl_w] & P_MASK_TILE == 0
							|| lvl[tx + (ty + 1) * lvl_w] & P_MASK_ONEWAY != 0));
//							&& lvl[tx - 1 + ty * lvl_w] & P_LVL_NO_TOP == 0));
					
					var slope1 = (tx <= 0 ? false : (lvl[tx - 1 + ty * lvl_w] & P_MASK_TILE == P_LVL_SLOPE_11_NW));
					var slope2 = (tx >= lvl_w - 1 ? false : (lvl[tx + 1 + ty * lvl_w] & P_MASK_TILE == P_LVL_SLOPE_11_NE));
					
					if (!coly && dy != 0
						&& cp.x >= (tx << 4) - 4 + (slope1 ? 4 : 0)
						&& cp.x < (tx << 4) + 21 - (slope2 ? 5 : 0)){
						if (chk_y1 && dy > 0 && cp.y == (ty << 4) - 7
								+ switch(lvl[tx + ty * lvl_w] & P_MASK_TILE){
									case P_LVL_SLOPE_11_NW:
									(16 - (cp.x - (tx << 4)) < 0 ? 0 : 16 - (cp.x - (tx << 4)));
									case P_LVL_SLOPE_11_NE:
									(cp.x - (tx << 4) < 0 ? 0 : cp.x - (tx << 4));
									case _: 0;
								}){
							e.coly1 = true;
							coly = true;
							cp.y -= 1;
							e.yv = 0;
						}
						if (chk_y2 && dy < 0 && cp.y == (ty << 4) + 20){
							e.coly2 = true;
							coly = true;
							cp.y += 1;
							e.yv = 0;
						}
					}
					if (!colx && dx != 0 && cp.y >= (ty << 4) - 7 && cp.y < (ty << 4) + 21){
						if (chk_x1 && dx > 0 && lvl[tx + ty * lvl_w] & P_MASK_TILE != P_LVL_SLOPE_11_NW && cp.x == (tx << 4) - 4){
							e.colx1 = true;
							colx = true;
							cp.x -= 1;
							e.xv = 0;
						} else if (chk_x1 && dx > 0 && lvl[tx + ty * lvl_w] & P_MASK_TILE == P_LVL_SLOPE_11_NW
							&& cp.x > (tx << 4) - (cp.y - (ty << 4)) + 6
							&& ((tx < lvl_w - 1 && ty > 0 && lvl[tx + 1 + (ty - 1) * lvl_w] & P_MASK_TILE == 0)
								? cp.y > (ty << 4) - 5 && cp.y < (ty << 4) + 30
								: true)
							){
							e.coly1 = true;
							cp.y -= cp.x - ((tx << 4) - (cp.y - (ty << 4)) + 6);
							e.yv = 0;
						}
						if (chk_x2 && dx < 0 && lvl[tx + ty * lvl_w] & P_MASK_TILE != P_LVL_SLOPE_11_NE && cp.x == (tx << 4) + 20){
							e.colx2 = true;
							colx = true;
							cp.x += 1;
							e.xv = 0;
						} else if (chk_x2 && dx < 0 && lvl[tx + ty * lvl_w] & P_MASK_TILE == P_LVL_SLOPE_11_NE
							&& cp.x < (tx << 4) + (cp.y - (ty << 4)) + 10){
							e.coly1 = true;
							cp.y -= ((tx << 4) + (cp.y - (ty << 4)) + 10) - cp.x;
							e.yv = 0;
						}
					}
				}
			}
			if (colx && coly) break;
			lp = cp;
		}
		e.xp = cp.x;
		e.yp = cp.y;
	}
}