package game;

import sk.thenet.plustd.FM;
import sk.thenet.gamestd.bmp.*;

class Tree {
	public var branches:Array<Branch>;
	public var cbd:BitmapData;
	public var pbbd:BitmapData;
	public var prog:Int = 0;
	public var phase:Int = 0;
	
	public function new(){
		cbd = new BitmapData(64, 64, 0);
		pbbd = new BitmapData(60, 2, 0);
		branches = [];
		branches.push(new Branch(0, 1, 0, -4, null, 0));
		update();
	}
	
	public function update():Void {
		for (b in branches){
			var bres = new Bresenham(FM.floor(b.sx), FM.floor(b.sy), FM.floor(b.ex), FM.floor(b.ey));
			for (p in bres){
				cbd.set(p.x + 32, p.y + 64, Assets.palette[b.kind * 5 + (switch (b.total_children){
					case (_ < 2) => true: 0;
					case (_ < 4) => true: 1;
					case (_ < 7) => true: 2;
					case (_ < 11) => true: 3;
					case _: 4;
				})]);
			}
		}
		/*
		for (b in branches){
			cbd.copyPixels(Assets.aEnemies, FM.floor(b.ex - 4) + 32, FM.floor(b.ey - 4) + 64, b.kind << 3, 40, 8, 8);
		}*/
	}
	
	public function render(bd:BitmapData, camxi:Int, camyi:Int){
		bd.copyPixels(cbd, 112 - camxi, 70 - camyi, 0, 0, cbd.width, cbd.height);
		bd.copyPixels(pbbd, 114 - camxi, 148 - camyi, 0, 0, pbbd.width, pbbd.height);
	}
	
	public function collect(kind:Int):Void {
		var tb = branches.filter(function(a:Branch){
			return a.total_children < 3;
		});
		tb.sort(function(a:Branch, b:Branch){
			return a.generation - b.generation;
		});
		var bb = tb[FM.floor(Math.random() * Math.random() * tb.length)];
		branches = branches.concat(bb.spawn(FM.floor(kind)));
		switch (kind){
			case 0:
			pbbd.set(prog * 2, 0, Assets.palette[12]);
			pbbd.set(prog *2, 1, Assets.palette[13]);
			case 1:
			pbbd.set(prog * 2, 0, Assets.palette[7]);
			pbbd.set(prog * 2, 1, Assets.palette[8]);
		}
		prog++;
		update();
	}
}

class Branch {
	public static inline var BT_ATTACK:Int = 0;
	public static inline var BT_EXPLORE:Int = 1;
	public static inline var BT_DEFEND:Int = 2;
	public static inline var BT_DEAD:Int = 3;
	
	public var children:Array<Branch> = [];
	public var total_children:Int = 0;
	public var parent:Branch;
	public var generation:Int = 0;
	public var sx:Float;
	public var sy:Float;
	public var ex:Float;
	public var ey:Float;
	public var kind:Int;
	public var mdirs:Array<Bool>;
	
	public function new(nsx:Float, nsy:Float, nex:Float, ney:Float, nparent:Branch, nkind:Int){
		kind = nkind;
		sx = nsx;
		sy = nsy;
		ex = nex;
		ey = ney;
		parent = nparent;
		mdirs = [false, false, false];
		if (parent != null){
			parent.addChild();
			generation = parent.generation + 1;
		}
	}
	
	public function addChild():Void {
		total_children++;
		if (parent != null){
			parent.addChild();
		}
	}
	
	public function spawn(kind:Int):Array<Branch> {
		var mdir:Float = Math.atan2(ey - sy, ex - sx);
		var si:Int = FM.floor(Math.random() * 3);
		for (i in 0...3){
			if (!mdirs[(si + i) % 3]) break;
			si += 1;
		}
		si %= 3;
		si--;
		var ndir:Float = mdir + si * .6 + Math.random() * .3 - .15;
		var len:Float = 4 + Math.random() * 3;
		children.push(new Branch(ex, ey, ex + Math.cos(ndir) * len, ey + Math.sin(ndir) * len, this, kind));
		return children;
	}
}